/* Copyright (c) 2006-2015 Polybus Systems Corp., Westford, MA, USA */

/* All rights reserved. */

/* Redistribution and use in source and binary forms, with or without modification, */
/* are permitted provided that the following conditions are met: */

/* * Redistributions of source code must retain the above copyright notice, this */
/* list of conditions and the following disclaimer. */
/* * Redistributions in binary form must reproduce the above copyright notice, */
/* this list of conditions and the following disclaimer in the documentation */
/* and/or other materials provided with the distribution. */
/* * Neither the name of the Polybus Systems Corp nor the names of its contributors */
/* may be used to endorse or promote products derived from this software */
/* without specific prior written permission. */

/* THIS SOFTWARE IS PROVIDED BY POLYBUS AND CONTRIBUTORS ''AS IS'' AND ANY */
/* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE */
/* DISCLAIMED. IN NO EVENT SHALL POLYBUS OR CONTRIBUTORS BE LIABLE FOR ANY */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES */
/* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; */
/* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON */
/* ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT */
/* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS */
/* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */

#include "sys_basher.h"

void *runTests(void *arg)
{
  SInt64 errors=0;
  thread_data *t= arg;
  SInt64 first;
  SInt64 limit = t->limit;
  void *base = t->base;
  SInt64 long_limit = t->long_limit;
  SInt32 select = t->test;
  SInt32 num = t->num;
  SInt32 vnum = t->vnum;
  SInt32 num_threads = t->num_threads;
  file_info *finfo = t->finfo;
  SInt32 verbose = t->verbose;

  switch(select)
    {
    case FIXED_MEM0:
    case FIXED_MEM1:
    case FIXED_MEM2:
    case FIXED_MEM3:
    case FIXED_MEM4:
    case FIXED_MEM5:
    case FIXED_MEM6:
    case FIXED_MEM7:
    case FIXED_MEM8:
    case FIXED_MEM9:
    case FIXED_MEM10:
    case FIXED_MEM11:
    case FIXED_MEM12:
    case FIXED_MEM13:
    case FIXED_MEM14:
    case FIXED_MEM15:
      if(!fixedTest(base,limit,num,select - FIXED_MEM0,t->rslt_summary->fixed,vnum,verbose))
	{
	  errors++;
	}
      break;
    case ADDR_EQ0:
    case ADDR_EQ1:
    case ADDR_EQ2:
    case ADDR_EQ3:
    case ADDR_EQ4:
    case ADDR_EQ5:
    case ADDR_EQ6:
    case ADDR_EQ7:
    case ADDR_EQ8:
    case ADDR_EQ9:
    case ADDR_EQ10:
    case ADDR_EQ11:
    case ADDR_EQ12:
    case ADDR_EQ13:
    case ADDR_EQ14:
    case ADDR_EQ15:
    case ADDR_EQ16:
    case ADDR_EQ17:
    case ADDR_EQ18:
    case ADDR_EQ19:
    case ADDR_EQ20:
    case ADDR_EQ21:
    case ADDR_EQ22:
    case ADDR_EQ23:
    case ADDR_EQ24:
    case ADDR_EQ25:
    case ADDR_EQ26:
    case ADDR_EQ27:
    case ADDR_EQ28:
    case ADDR_EQ29:
    case ADDR_EQ30:
    case ADDR_EQ31:
      if(!addrEqDataTest(base,limit,num,select - ADDR_EQ0,t->rslt_summary->addreq,vnum,verbose))
	{
	  errors++;
	}
      break;
    case BANK_TST2:
    case BANK_TST3:
    case BANK_TST4:
    case BANK_TST5:
    case BANK_TST6:
    case BANK_TST7:
    case BANK_TST8:
    case BANK_TST9:
    case BANK_TST10:
    case BANK_TST11:
    case BANK_TST12:
    case BANK_TST13:
    case BANK_TST14:
    case BANK_TST15:
      if(!bankTest(base,limit,num,(select - BANK_TST2) + 2,t->rslt_summary->bank,vnum,verbose))
	{
	  errors++;
	}
      break;
    case REV_ADDR0:
      if(!revAddressTest(base,limit,num,select - REV_ADDR0,t->rslt_summary->rev_addr,vnum,verbose))
	{
	  errors++;
	}
      break;
    case REV_ADDR1:
      if(!revAddressTopDownTest(base,limit,num,select - REV_ADDR0,t->rslt_summary->rev_addr,vnum,verbose))
	{
	  errors++;
	}
      break;
    case LFSR_TST0:
      if(!lfsrTest(base,limit,num,select - LFSR_TST0,t->rslt_summary->lfsr,vnum,verbose,t->seed,num_threads == 1))
	{
	  errors++;
	}
      break;
    case LFSR_TST1:
    case LFSR_TST2:
    case LFSR_TST3:
    case LFSR_TST4:
    case LFSR_TST5:
    case LFSR_TST6:
    case LFSR_TST7:
      if(!lfsrTest(base,limit,num,select - LFSR_TST0,t->rslt_summary->lfsr,vnum,verbose,t->seed,FALSE))
	{
	  errors++;
	}
      break;
    case WALK_TST0:
    case WALK_TST1:
    case WALK_TST2:
    case WALK_TST3:
    case WALK_TST4:
    case WALK_TST5:
    case WALK_TST6:
    case WALK_TST7:
      if(!walkTest(base,limit,num,select - WALK_TST0,t->rslt_summary->walk,vnum,verbose))
	{
	  errors++;
	}
      break;
    case MEM_BAND:
      if(!memBandwidth(base,limit,num,num_threads,t->rslt_summary->mem_bandwidth,vnum))
	{
	  errors++;
	}
      break;
    case FP_TST:
      first = t->firsttime;
      if(!fpFloatTest(base,limit,num,num_threads,first,t->expected->fp,t->rslt_summary->sgl_rslt,vnum,t->expected->time_scale_fp))
	{
	  errors++;
	}

      if(!fpDoubleTest(base,limit,num,num_threads,first,t->expected->dbl,t->rslt_summary->dbl_rslt,vnum,t->expected->time_scale_dbl))
	{
	  errors++;
	}
      break;
    case INT_TST:
      first = t->firsttime;
      if(!intSInt16Test(base,limit,num,num_threads,first,t->expected->int16,t->rslt_summary->int16_rslt,vnum,t->expected->time_scale16))
	{
	  errors++;
	}

      if(!intLongTest(base,limit,num,num_threads,first,t->expected->int32,t->rslt_summary->int32_rslt,vnum,t->expected->time_scale32))
	{
	  errors++;
	}

      if(!intSInt64Test(base,limit,num,num_threads,first,t->expected->int64,t->rslt_summary->int64_rslt,vnum,t->expected->time_scale64))
	{
	  errors++;
	}
      break;
    case DISK_TST:
      if(!fileTest((char *)base,num,num_threads,finfo,limit,t->rslt_summary->disk_rslt,vnum))
	{
	  errors++;
	}
      break;
    case SWAP_TST:
      if(!swapTest(base,long_limit,num,t->rslt_summary->swap,vnum,verbose))
	{
	  errors++;
	}
      break;
    }
  t->errors = errors;
  return(NULL);
}
